/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Josh Aas <josh@mozilla.com>
 *  Thomas K. Dyas <tom.dyas@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef _U_GLOBALMENUDOCLISTENER_H
#define _U_GLOBALMENUDOCLISTENER_H

#include <nsIDOMMutationObserver.h>
#include <nsAutoPtr.h>
#include <nsHashKeys.h>
#include <nsClassHashtable.h>

#ifdef DEBUG
#define DEBUG_chrisccoulson
#endif

class nsIContent;
class uGlobalMenuObject;
class nsAString;

class uGlobalMenuDocListener: public nsIMutationObserverCallback
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIMUTATIONOBSERVERCALLBACK

  uGlobalMenuDocListener();
  nsresult Init(nsIContent *rootNode);
  void RegisterForContentChanges(nsIContent *aContent,
                                 uGlobalMenuObject *aMenuObject);
  void UnregisterForContentChanges(nsIContent *aContent,
                                   uGlobalMenuObject *aMenuObject);
  void Destroy();
  virtual ~uGlobalMenuDocListener();

  void HandleMutations(nsTArray<nsCOMPtr<nsIDOMMutationRecord> >& aRecords);

  static void EnterCriticalZone() { sInhibitDepth++; }
  static void LeaveCriticalZone();

  static void Shutdown();

private:
  void AttributeChanged(nsIContent *aContent, nsAString& aAttribute);
  void ContentRemoved(nsIContent *aContainer, nsIContent *aChild);
  void ContentInserted(nsIContent *aContainer, nsIContent *aChild,
                       nsIContent *aPrevSibling);

  nsTArray<uGlobalMenuObject *>* GetListenersForContent(nsIContent *aContent,
                                                        bool aCreate);

  void FlushPendingMutations();

  nsCOMPtr<nsIDOMMutationObserver> mObserver;
  nsClassHashtable<nsPtrHashKey<nsIContent>, nsTArray<uGlobalMenuObject *> > mContentToObserverTable;

  nsTArray<nsCOMPtr<nsIDOMMutationRecord> > mPendingMutations;

  static void ScheduleListener(uGlobalMenuDocListener *aListener);

  static uint32_t sInhibitDepth;
  static nsTArray<nsCOMPtr<uGlobalMenuDocListener> > *sPendingListeners;
};

class uMenuAutoSuspendMutationEvents
{
public:
  uMenuAutoSuspendMutationEvents()
  {
    uGlobalMenuDocListener::EnterCriticalZone();
  }

  ~uMenuAutoSuspendMutationEvents()
  {
    uGlobalMenuDocListener::LeaveCriticalZone();
  }
};

#endif
